      program blob
c	**************************************************************
c        program follows a photon through a spherical blob.
c
c	 standard Monte Carlo method described in Code & Whitney 1995,
c	 	ApJ, 
c
c
c Written by Barbara A Whitney and Arthur D. Code, Washburn Observatory
c
c modification history:
c 87/05/19 (baw)
c 87/05/21 (adc)
c 88/07/05 (adc)
c 90/04/01 (adc)
c 90/07/16 (baw)
c 92/11/19 (baw): fixed flux norm
c 92/11/25 (adc): proper normalization is Fi = Ni/2pi Ntotal Sin (theta)
c                 d(theta), since d(theta) = pi/binc in radians then the
c                 normalized flux is ti(l)/2.*pi^2*aphot*sin(theta(l)/binc
c                 in the program parameters.
c 93/05/25 (adc)
c 95/01/16 (mjw)  change parameter method.  lose namelist and using
c                   straight ascii file (by REAPAR).
c                 add GETSET,ERRMSG, and WRIMSG to help without output.
c 95/02/22 (baw)  fixed an error in diffuse incident light--photons
c	          had the right incident angle but didn't go into the
c	          proper position.
c 96/12/08 (baw)  include arbitrary phase functions, e.g. input from
c		  mie calculations
c 00/08/08 (baw)  ported to linux
c	
c	***************************************************************

      byte state(8)

      real rad(10)

      real s

      integer IDVOUT,IDVERR,iopar
      character cflpar*20,cmsgnm*70

      include 'params.txt'
      include 'arrays.txt'
      include 'blob.txt'
      include 'stokes.txt'
      include 'mrncoef.txt'
      include 'tab.txt'
	include 'random.txt'
      
      external errmsg,getset,reapar,wrimsg

      data pi /3.14159265/
      data IDVOUT/0/,IDVERR/0/,iopar/11/
      data cflpar/'blob.par'/

c ... set logical unit numbers of calls to WRIMSG and ERRMSG
      CALL GETSET('SET','IOOUT',IDVOUT)
      CALL GETSET('SET','IOERR',IDVERR)

c ... load parameters.  Those not passed as arguments are
c     in common blocks.
      call REAPAR(cflpar,chi,dangle,dens,i1,iopar,ixy,
     $ rblob,rsurf,x10,y10)

c ... define some useful constants
      g2=g**2
      hit=0
      htot=0
      r2p=2.*pi

c     initialize random number generator
c      call initrand(i1,state)
c      call setrand(state)


c	initialize arrays and counters
	call iarray

	rblob2=rblob*rblob

c	blob is illuminated from -z direction.  so mu0=1.
c	z is defined to be zero at bottom of sphere.  so
c	center of sphere is at z=rblob.
c	note that x**2+y**2+(z-rblob)**2=rblob**2
c	so x**2+y**2+z**2=2.*z*rblob.
c	and z-rblob=+/-sqrt(rblob**2-x**2-y**2)
c	also note that we take advantage of azimuthal symmetry.
c	summing over phi is like integrating over longitude.
	
c	test for uniform illumination
c	do i=1,5
c	  xtest=(float(i-1)*2)/10
c	  this is wrong.
c	  rad(i)=xtest
c	  this is right.
c	  rad(i)=sqrt(xtest)
c	end do
c	rtest=photons/4
	totscat=0

	
c	************* do loop over each photon ***********************

      do i=1,photons
         if (mod(i,100000).eq.0) then
            write(cmsgnm,'(i12,a20)')i,
     $           ' photons completed.'
            call WRIMSG('BLOB96',cmsgnm)
         endif
c	   if (i.gt.15000) print*,i
c	   if (i.eq.15516) then
c            continue
c         endif
         if (ixy.eq.0) then
c     	    uniform illumination.
c           since we integrate over phi, don't have to distribute
c           incident light over phi.  only needs radial dependence.
c	    note: this must change when we include imaging!
          xran=ran2(i1)
	    x1=sqrt(xran)*rblob
	    y1=0.
c           test...
c           itest=(float(i)-0.5)/rtest+1 
c           x1=rad(itest)*rblob
c           y1=0.
         else
c           in units of rblob
	    x1=x10*rblob
	    y1=y10*rblob
         end if
         z1=rblob-sqrt(rblob2-x1**2-y1**2)
         xran=ran2(i1)         
         cost=cos(dangle*(xran))
         sint=sin(dangle*(xran))
c	 sample phi
       xran=ran2(i1)         
	 cosp=cos(r2p*xran)
	 sinp=sin(r2p*xran)
       xran=ran2(i1)         
	 tau=-alog(xran)
	 s=tau/chi
	 ztot=z1+s*cost
         xtot=x1+s*sint*cosp
         ytot=y1+s*sint*sinp
         if (ztot.lt.0.) then
           call errmsg('WARNING','BLOB96', 
     $     'something wrong w/ initial cond')
	 end if
         nscat=0
         tod=xtot**2+ytot**2+ztot**2
c         phi=0.
         sip=1.
         sqp=0.
         sup=0.
         svp=0.
         aflag=0
         
         tempscat=0.
c     photon scatters at random angles and tau's until it exits
         do while (tod .lt. 2.*rblob*ztot)
c	 that is, x^2+y^2+(z1-r)^2 < r^2, since z1-r=0 at middle of blob
	    nscat=nscat+1
          xran=ran2(i1)         
	    if(xran.lt.rlam) then
               tempscat=tempscat+1
	       if(idust.lt.2) then
                 call stokes
	       else
	         call stokes6
	       end if
	    else
               aflux=aflux+1
               aflag=1
               go to 20
	    end if
          xran=ran2(i1)         
	    tau=-alog(xran)
	    s=tau/chi
	    zd=s*cost
	    xd=s*sint*cosp
	    yd=s*sint*sinp
	    ztot=ztot+zd	
	    xtot=xtot+xd
	    ytot=ytot+yd
	    tod=xtot**2+ytot**2+ztot**2
         end do

 20      continue
         if (nscat.eq.0) then
	    ncheck=ncheck+1
         else
	    if(aflag.eq.0) then
               totscat=totscat+tempscat
               thet=acos(cost)
               rnum=binc*thet/pi
               l=int(rnum)+1
               erro(l)=erro(l)+1.
               ti(l)=ti(l)+sip
               tq(l)=tq(l)+sqp
               tu(l)=tu(l)+sup
               tv(l)=tv(l)+svp
               ti2(l)=ti2(l)+sip*sip
               tq2(l)=tq2(l)+sqp*sqp
               tu2(l)=tu2(l)+sup*sup
               tv2(l)=tv2(l)+svp*svp
	    end if
         end if
      end do
      
c     *******************************************************


      call WRIMSG('BLOB96','done with atmosphere')
      
c     put results into intensities.
      call intensity
      
c     output files
      call output(ixy,y10,x10)
      

 999  continue
      end
