      SUBROUTINE REAPAR(cflpar,chi,dangle,dens,i1,iopar,ixy,
     $ rblob,rsurf,x10,y10)
C***********************************************************************
C Subroutine REAPAR handles the reading of input parameters from the
C ".par" file, as well as elementary processing with those input
C parameters to generate arrays.
c
C Original version created by P.J.Flatau, Colorado State Univ.
C Modified by B.T.Draine, Princeton Univ. Obs.
C ported from program DDSCAT by m. wolff, Washburn Observatory
c
C History:
C 95/01/17 (mjw): stripped out of DDSCAT and ported to BLOB
C
C***********************************************************************
C
C     .. Scalar Arguments ..
      real dangle,x10,y10,rblob,rsurf,dens,kappa,chi,rlamtmp
      integer i1,iopar,ixy
      character cflout*60,cflpar*(*)
C     ..
C     .. Local Scalars ..
      character cigeom*5,cline*70,cmsgnm*70,cscat*5,copt*3,cfile*50
C     ..
C     .. Local Arrays ..
C     ..
C     .. External Subroutines ..
      external errmsg, wrimsg
C     ..
C     .. Intrinsic Functions ..
C     ..
C     .. Data statements ..
C     ..
c     .. Include statements ..
      include 'params.txt'
      include 'arrays.txt'
      include 'blob.txt'
      include 'mrncoef.txt'
      include 'stokes.txt'
      include 'tab.txt'
C***********************************************************************
      open(unit=iopar,file=cflpar,status='old')

      READ(IOPAR,FMT=*,ERR=40)CLINE
      CALL WRIMSG(' ',CLINE)
      READ(IOPAR,FMT=*,ERR=40)CLINE
      CALL WRIMSG(' ',CLINE)
C***********************************************************************
C
c     PHOTONS = number of photons to be used in computation
c     I1 = random number seed
c     BINC = bins in theta in output (note: this a real number)
c
      READ(IOPAR,FMT=*,ERR=40)photons
      WRITE(CMSGNM,FMT='(i11,A)')photons
     $    ,' = number of photons'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)i1
      WRITE(CMSGNM,FMT='(i10,A)')i1
     $    ,' = random number seed'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)binc
      WRITE(CMSGNM,FMT='(f10.0,A)')binc
     $    ,' = number of bins in theta (output)'
      CALL WRIMSG('REAPAR',CMSGNM)
C***********************************************************************
C
c     CIGEOM*5 = UNIFO (uniform geometry, ixy=0)
c                USER1 (specify inital photon location, ixy=1)
c     DANGLE   = size of illuminating source (in radians)
c     X10   = initial x coordinate of incoming photon (if ixy=1)
c     Y10   = initial y coordinate of incoming photon (if ixy=1)
c     TNOT  = optical depth of uniform medium hence also
c             geometrical depth in some appropriate units.
c
      READ(IOPAR,FMT=*,ERR=40)CLINE
      CALL WRIMSG(' ',CLINE)
      READ(IOPAR,FMT=*,ERR=40)CLINE
      READ(IOPAR,FMT=*,ERR=40)cigeom
      WRITE(CMSGNM,FMT='(A,A)')
     $    cigeom,' - illumination geometry'
      CALL WRIMSG('REAPAR',CMSGNM)
      IF(cigeom.ne.'UNIFO'.AND.cigeom.ne.'USER1') then
         CALL ERRMSG('FATAL','REAPAR',
     $        'Check value of CIGEOM*5 in .PAR file')
      else
         if(cigeom.eq.'UNIFO') ixy=0
         if(cigeom.eq.'USER1') ixy=1
      endif
      READ(IOPAR,FMT=*,ERR=40)dangle
      WRITE(CMSGNM,FMT='(1pe11.3,a)')dangle
     $     ,' = size illumination source (radians)'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)x10
      READ(IOPAR,FMT=*,ERR=40)y10
      if(cigeom.eq.'USER1') then
         WRITE(CMSGNM,FMT='(f10.2,a)')x10
     $        ,' = inital x-coord for photons'
         CALL WRIMSG('REAPAR',CMSGNM)
         WRITE(CMSGNM,FMT='(f10.2,A)')y10
     $        ,' = inital y-coord for photons'
         CALL WRIMSG('REAPAR',CMSGNM)
      endif
C***********************************************************************
C
c     COPT*3 =   TAU (optical depth, R=1)
c                NUM (number density, specify R)
c     RBLOB =  radius of outer atmosphere
c     RSURF =  radius of hard surface of planet 
c     TNOT  =  optical depth of uniform medium (for COPT='TAU')
c     DENS  =  particle density  (for COPT='NUM')
c
      READ(IOPAR,FMT=*,ERR=40)CLINE
      CALL WRIMSG(' ',CLINE)
      READ(IOPAR,FMT=*,ERR=40)copt
      WRITE(CMSGNM,FMT='(A,A)')
     $    copt,' - optical parameters'
      CALL WRIMSG('REAPAR',CMSGNM)
      IF(copt.ne.'TAU'.AND.copt.ne.'NUM') then
         CALL ERRMSG('FATAL','REAPAR',
     $        'Check value of COPT*3 in .PAR file')
      endif
      READ(IOPAR,FMT=*,ERR=40)rblob
      WRITE(CMSGNM,FMT='(1pe11.3,a)')rblob
     $     ,' = radius of outer atmosphere'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)rsurf
      WRITE(CMSGNM,FMT='(1pe11.3,a)')rsurf
     $     ,' = radius of surface'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)tnot
      if(copt.eq.'TAU') then
         WRITE(CMSGNM,FMT='(f10.2,a)')tnot
     $        ,' = optical depth from surface to outer atmosphere'
         CALL WRIMSG('REAPAR',CMSGNM)
      endif
      READ(IOPAR,FMT=*,ERR=40)dens
      if(copt.eq.'NUM') then
         WRITE(CMSGNM,FMT='(f10.2,a)')dens
     $        ,' = number density of particles'
         CALL WRIMSG('REAPAR',CMSGNM)
      endif
C***********************************************************************
C
c     CSCAT*5 =  THOMP (Thompson scattering, idust=0)
c                DUST1 (Dust scattering, idust=1)
c		 TABLE (read in from table )
c     RLAM    =  albedo = scattering/absorption
c     G       =  dust scattering asymmetry factor
c     PL      =  peak value of linear polarization
c     PC      =  peak value of linear to circular conversion
c     SC      =  asymmetry of the circular polarization
c
c      rlam=1.00 idust=0  g=.56     pl=.51  pc=.39  sc=1.
c      dangle=1.00
C
c
      READ(IOPAR,FMT=*,ERR=40)CLINE
      CALL WRIMSG(' ',CLINE)
      READ(IOPAR,FMT=*,ERR=40)cscat
      WRITE(CMSGNM,FMT='(A,A)')
     $    cscat,' - type of scattering'
      CALL WRIMSG('REAPAR',CMSGNM)
      IF(cscat.ne.'DUST1'.AND.cscat.ne.'THOMP'.and.cscat.ne.'TABLE')then
         CALL ERRMSG('FATAL','REAPAR',
     $        'Check value of CSCAT*5 in .PAR file')
      else
         if(cscat.eq.'THOMP') then
            idust=0
	      rlam=1.
         else if(cscat.eq.'DUST1') then
            idust=1
         endif

      endif
      READ(IOPAR,FMT=*,ERR=40)cfile
      READ(IOPAR,FMT=*,ERR=40) nelems
      if(cscat.eq.'TABLE') then
         WRITE(CMSGNM,FMT='(A,A)')
     $    cfile,' - name of file'
         CALL WRIMSG('REAPAR',CMSGNM)
	   WRITE(CMSGNM,FMT=*) nelems
     $    ,' = number of unique elements in scattering matrix'
         CALL WRIMSG('REAPAR',CMSGNM)
	   idust=2
c	   here's where you put in your favorite subroutine to read in your
c	   favorite table of scattering properties.  
         CALL REATAB(cfile,rlam)
      end if

      if(idust.lt.0.or.idust.gt.2) then
        CALL ERRMSG('FATAL','REAPAR',
     $        'idust out of range')
      end if
      READ(IOPAR,FMT=*,ERR=40)kappa
      READ(IOPAR,FMT=*,ERR=40)rlamtmp
	if (cscat.eq.'DUST1') rlam=rlamtmp
      READ(IOPAR,FMT=*,ERR=40)g
      READ(IOPAR,FMT=*,ERR=40)pl
      READ(IOPAR,FMT=*,ERR=40)pc
      READ(IOPAR,FMT=*,ERR=40)sc
       WRITE(CMSGNM,FMT='(f11.3,a)')rlam
     $      ,' = albedo'
       CALL WRIMSG('REAPAR',CMSGNM)
      if(copt.eq.'NUM') then
       WRITE(CMSGNM,FMT='(f11.3,a)')kappa
     $      ,' = cross section'
       CALL WRIMSG('REAPAR',CMSGNM)
      endif
      if(cscat.eq.'DUST1'.or.cscat.eq.'THOMP') then
       WRITE(CMSGNM,FMT='(f10.2,A)')pl
     $      ,' = peak of linear polarization'
       CALL WRIMSG('REAPAR',CMSGNM)
      end if



      if(cscat.eq.'DUST1') then
         WRITE(CMSGNM,FMT='(f11.3,a)')g
     $        ,' = dust asymmetry factor'
         CALL WRIMSG('REAPAR',CMSGNM)
         WRITE(CMSGNM,FMT='(f10.2,A)')pc
     $        ,' = peak of conversion from linear to circular'
         CALL WRIMSG('REAPAR',CMSGNM)
         WRITE(CMSGNM,FMT='(f10.2,A)')sc
     $        ,' = asymmetry of circular polarization'
         CALL WRIMSG('REAPAR',CMSGNM)
      endif
C
      if(copt.eq.'NUM') then
	chi=kappa*dens
      else
	chi=tnot/(rblob-rsurf)
      endif

	if (idust.eq.0) then
         peak=2.0
	else if (idust.eq.1) then
         peak=(1.-g**2)/(1.+g**2-2*g)**1.5
	   if (peak.lt.2) peak=2.     !must be in mult. scattering...
c        Cornette & Shanks modified H-G function
         peak=1.5*(1.-g**2)/(2.+g**2)*2./(1.+g**2-2.*g)**1.5
	endif

      WRITE(CMSGNM,FMT='(f11.3,a)')peak
     $     ,' = peak of normalized phase function'
      CALL WRIMSG('REAPAR',CMSGNM)


      CLOSE (IOPAR)
      RETURN
   40 CONTINUE
      CALL ERRMSG('FATAL','REAPAR',' Error reading .par file')
      RETURN
      END
